@@ -1,6 +1,9 @@
 use crate::color::{ColoredString, Colors, Elem};
+use crate::flags::HyperlinkOption;
 use crate::icon::Icons;
 use crate::meta::filetype::FileType;
+use crate::print_error;
+use crate::url::Url;
 use std::cmp::{Ordering, PartialOrd};
 use std::ffi::OsStr;
 use std::path::{Component, Path, PathBuf};
@@ -96,25 +99,69 @@ impl Name {
         }
     }
 
+    fn hyperlink(&self, name: String, hyperlink: HyperlinkOption) -> String {
+        match hyperlink {
+            HyperlinkOption::Always => {
+                // HyperlinkOption::Auto gets converted to None or Always in core.rs based on tty_available
+                match std::fs::canonicalize(&self.path) {
+                    Ok(rp) => {
+                        match Url::from_file_path(&rp) {
+                            Ok(url) => {
+                                // Crossterm does not support hyperlinks as of now
+                                // https://gist.github.com/egmontkob/eb114294efbcd5adb1944c9f3cb5feda
+                                format!("\x1B]8;;{}\x1B\x5C{}\x1B]8;;\x1B\x5C", url, name)
+                            }
+                            Err(_) => {
+                                print_error!("{}: unable to form url.", name);
+                                name
+                            }
+                        }
+                    }
+                    Err(err) => {
+                        match err.kind() {
+                            std::io::ErrorKind::NotFound => {
+                                // If this happens, it just means the file is a broken symlink. This is not an error, and the user is already warned that the symlink is broken by the colors.
+                                name
+                            }
+                            _ => {
+                                print_error!("{}: {}", name, err);
+                                name
+                            }
+                        }
+                    }
+                }
+            }
+            _ => name,
+        }
+    }
+
     pub fn render(
         &self,
         colors: &Colors,
         icons: &Icons,
         display_option: &DisplayOption,
+        hyperlink: HyperlinkOption,
     ) -> ColoredString {
         let content = match display_option {
             DisplayOption::FileName => {
-                format!("{}{}", icons.get(self), self.escape(self.file_name()))
+                format!(
+                    "{}{}",
+                    icons.get(self),
+                    self.hyperlink(self.escape(self.file_name()), hyperlink)
+                )
             }
             DisplayOption::Relative { base_path } => format!(
                 "{}{}",
                 icons.get(self),
-                self.escape(&self.relative_path(base_path).to_string_lossy())
+                self.hyperlink(
+                    self.escape(&self.relative_path(base_path).to_string_lossy()),
+                    hyperlink
+                )
             ),
             DisplayOption::None => format!(
                 "{}{}",
                 icons.get(self),
-                self.escape(&self.path.to_string_lossy())
+                self.hyperlink(self.escape(&self.path.to_string_lossy()), hyperlink)
             ),
         };
 
@@ -166,11 +213,13 @@ mod test {
     use super::DisplayOption;
     use super::Name;
     use crate::color::{self, Colors};
+    use crate::flags::HyperlinkOption;
     use crate::icon::{self, Icons};
     use crate::meta::FileType;
     use crate::meta::Meta;
     #[cfg(unix)]
     use crate::meta::Permissions;
+    use crate::url::Url;
     use crossterm::style::{Color, Stylize};
     use std::cmp::Ordering;
     use std::fs::{self, File};
@@ -198,7 +247,12 @@ mod test {
 
         assert_eq!(
             " file.txt".to_string().with(Color::AnsiValue(184)),
-            name.render(&colors, &icons, &DisplayOption::FileName)
+            name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
     }
 
@@ -207,7 +261,7 @@ mod test {
         let tmp_dir = tempdir().expect("failed to create temp dir");
         let icons = Icons::new(icon::Theme::Fancy, " ".to_string());
 
-        // Chreate the directory
+        // Create the directory
         let dir_path = tmp_dir.path().join("directory");
         fs::create_dir(&dir_path).expect("failed to create the dir");
         let meta = Meta::from_path(&dir_path, false).unwrap();
@@ -216,7 +270,12 @@ mod test {
 
         assert_eq!(
             " directory".to_string().with(Color::AnsiValue(33)),
-            meta.name.render(&colors, &icons, &DisplayOption::FileName)
+            meta.name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
     }
 
@@ -244,7 +303,12 @@ mod test {
 
         assert_eq!(
             " target.tmp".to_string().with(Color::AnsiValue(44)),
-            name.render(&colors, &icons, &DisplayOption::FileName)
+            name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
     }
 
@@ -272,7 +336,12 @@ mod test {
 
         assert_eq!(
             " target.d".to_string().with(Color::AnsiValue(44)),
-            name.render(&colors, &icons, &DisplayOption::FileName)
+            name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
     }
 
@@ -298,7 +367,12 @@ mod test {
 
         assert_eq!(
             " pipe.tmp".to_string().with(Color::AnsiValue(184)),
-            name.render(&colors, &icons, &DisplayOption::FileName)
+            name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
     }
 
@@ -317,7 +391,45 @@ mod test {
         assert_eq!(
             "file.txt",
             meta.name
-                .render(&colors, &icons, &DisplayOption::FileName)
+                .render(
+                    &colors,
+                    &icons,
+                    &DisplayOption::FileName,
+                    HyperlinkOption::Never
+                )
+                .to_string()
+                .as_str()
+        );
+    }
+
+    #[test]
+    fn test_print_hyperlink() {
+        let tmp_dir = tempdir().expect("failed to create temp dir");
+        let icons = Icons::new(icon::Theme::NoIcon, " ".to_string());
+
+        // Create the file;
+        let file_path = tmp_dir.path().join("file.txt");
+        File::create(&file_path).expect("failed to create file");
+        let meta = Meta::from_path(&file_path, false).unwrap();
+
+        let colors = Colors::new(color::ThemeOption::NoColor);
+
+        let real_path = std::fs::canonicalize(&file_path).expect("canonicalize");
+        let expected_url = Url::from_file_path(&real_path).expect("absolute path");
+        let expected_text = format!(
+            "\x1B]8;;{}\x1B\x5C{}\x1B]8;;\x1B\x5C",
+            expected_url, "file.txt"
+        );
+
+        assert_eq!(
+            expected_text,
+            meta.name
+                .render(
+                    &colors,
+                    &icons,
+                    &DisplayOption::FileName,
+                    HyperlinkOption::Always
+                )
                 .to_string()
                 .as_str()
         );
@@ -533,7 +645,12 @@ mod test {
 
         assert_eq!(
             " file\\ttab.txt".to_string().with(Color::AnsiValue(184)),
-            name.render(&colors, &icons, &DisplayOption::FileName)
+            name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
 
         let file_path = tmp_dir.path().join("file\nnewline.txt");
@@ -548,7 +665,12 @@ mod test {
             " file\\nnewline.txt"
                 .to_string()
                 .with(Color::AnsiValue(184)),
-            name.render(&colors, &icons, &DisplayOption::FileName)
+            name.render(
+                &colors,
+                &icons,
+                &DisplayOption::FileName,
+                HyperlinkOption::Never
+            )
         );
     }
 }
